@@ -7,7 +7,7 @@
 //
 //------------------------------------------------------------------------------
 //
-//   Copyright 2014 - 2018    Michael Griffin    <m12.griffin@gmail.com>
+//   Copyright 2014 - 2020    Michael Griffin    <m12.griffin@gmail.com>
 //
 //   Licensed under the Apache License, Version 2.0 (the "License");
 //   you may not use this file except in compliance with the License.
@@ -45,9 +45,16 @@
 #include "sub_simd_x86.h"
 #endif
 
-#ifdef AF_HASSIMD_ARM
+#if defined(AF_HASSIMD_ARMv7_32BIT) || defined(AF_HASSIMD_ARM_AARCH64)
 #include "arm_neon.h"
-#include "sub_simd_arm.h"
+#endif
+
+#if defined(AF_HASSIMD_ARMv7_32BIT)
+#include "sub_simd_armv7.h"
+#endif
+
+#if defined(AF_HASSIMD_ARM_AARCH64)
+#include "sub_simd_armv8.h"
 #endif
 
 
@@ -72,7 +79,7 @@ signed int sub_signed_char_1(Py_ssize_t arraylen, int nosimd, signed char *data1
 	// Math error checking disabled.
 	if (ignoreerrors) {
 
-#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARM)
+#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARMv7_32BIT) || defined(AF_HASSIMD_ARM_AARCH64)
 		// SIMD version.
 		if (!nosimd && (arraylen >= (CHARSIMDSIZE * 2))) {
 			sub_signed_char_1_simd(arraylen, data1, param);
@@ -118,7 +125,7 @@ signed int sub_signed_char_2(Py_ssize_t arraylen, int nosimd, signed char *data1
 	// Math error checking disabled.
 	if (ignoreerrors) {
 
-#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARM)
+#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARMv7_32BIT) || defined(AF_HASSIMD_ARM_AARCH64)
 		// SIMD version.
 		if (!nosimd && (arraylen >= (CHARSIMDSIZE * 2))) {
 			sub_signed_char_2_simd(arraylen, data1, param, data3);
@@ -165,7 +172,7 @@ signed int sub_signed_char_3(Py_ssize_t arraylen, int nosimd, signed char param,
 	// Math error checking disabled.
 	if (ignoreerrors) {
 
-#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARM)
+#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARMv7_32BIT) || defined(AF_HASSIMD_ARM_AARCH64)
 		// SIMD version.
 		if (!nosimd && (arraylen >= (CHARSIMDSIZE * 2))) {
 			sub_signed_char_3_simd(arraylen, param, data2);
@@ -214,7 +221,7 @@ signed int sub_signed_char_4(Py_ssize_t arraylen, int nosimd, signed char param,
 	// Math error checking disabled.
 	if (ignoreerrors) {
 
-#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARM)
+#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARMv7_32BIT) || defined(AF_HASSIMD_ARM_AARCH64)
 		// SIMD version.
 		if (!nosimd && (arraylen >= (CHARSIMDSIZE * 2))) {
 			sub_signed_char_4_simd(arraylen, param, data2, data3);
@@ -264,7 +271,7 @@ signed int sub_signed_char_5(Py_ssize_t arraylen, int nosimd, signed char *data1
 	// Math error checking disabled.
 	if (ignoreerrors) {
 
-#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARM)
+#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARMv7_32BIT) || defined(AF_HASSIMD_ARM_AARCH64)
 		// SIMD version.
 		if (!nosimd && (arraylen >= (CHARSIMDSIZE * 2))) {
 			sub_signed_char_5_simd(arraylen, data1, data2);
@@ -296,7 +303,7 @@ signed int sub_signed_char_6(Py_ssize_t arraylen, int nosimd, signed char *data1
 	// Math error checking disabled.
 	if (ignoreerrors) {
 
-#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARM)
+#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARMv7_32BIT) || defined(AF_HASSIMD_ARM_AARCH64)
 		// SIMD version.
 		if (!nosimd && (arraylen >= (CHARSIMDSIZE * 2))) {
 			sub_signed_char_6_simd(arraylen, data1, data2, data3);
@@ -337,7 +344,7 @@ signed int sub_unsigned_char_1(Py_ssize_t arraylen, int nosimd, unsigned char *d
 	// Math error checking disabled.
 	if (ignoreerrors) {
 
-#if defined(AF_HASSIMD_ARM)
+#if defined(AF_HASSIMD_ARMv7_32BIT) || defined(AF_HASSIMD_ARM_AARCH64)
 		// SIMD version.
 		if (!nosimd && (arraylen >= (CHARSIMDSIZE * 2))) {
 			sub_unsigned_char_1_simd(arraylen, data1, param);
@@ -368,7 +375,7 @@ signed int sub_unsigned_char_2(Py_ssize_t arraylen, int nosimd, unsigned char *d
 	// Math error checking disabled.
 	if (ignoreerrors) {
 
-#if defined(AF_HASSIMD_ARM)
+#if defined(AF_HASSIMD_ARMv7_32BIT) || defined(AF_HASSIMD_ARM_AARCH64)
 		// SIMD version.
 		if (!nosimd && (arraylen >= (CHARSIMDSIZE * 2))) {
 			sub_unsigned_char_2_simd(arraylen, data1, param, data3);
@@ -399,7 +406,7 @@ signed int sub_unsigned_char_3(Py_ssize_t arraylen, int nosimd, unsigned char pa
 	// Math error checking disabled.
 	if (ignoreerrors) {
 
-#if defined(AF_HASSIMD_ARM)
+#if defined(AF_HASSIMD_ARMv7_32BIT) || defined(AF_HASSIMD_ARM_AARCH64)
 		// SIMD version.
 		if (!nosimd && (arraylen >= (CHARSIMDSIZE * 2))) {
 			sub_unsigned_char_3_simd(arraylen, param, data2);
@@ -430,7 +437,7 @@ signed int sub_unsigned_char_4(Py_ssize_t arraylen, int nosimd, unsigned char pa
 	// Math error checking disabled.
 	if (ignoreerrors) {
 
-#if defined(AF_HASSIMD_ARM)
+#if defined(AF_HASSIMD_ARMv7_32BIT) || defined(AF_HASSIMD_ARM_AARCH64)
 		// SIMD version.
 		if (!nosimd && (arraylen >= (CHARSIMDSIZE * 2))) {
 			sub_unsigned_char_4_simd(arraylen, param, data2, data3);
@@ -463,7 +470,7 @@ signed int sub_unsigned_char_5(Py_ssize_t arraylen, int nosimd, unsigned char *d
 	// Math error checking disabled.
 	if (ignoreerrors) {
 
-#if defined(AF_HASSIMD_ARM)
+#if defined(AF_HASSIMD_ARMv7_32BIT) || defined(AF_HASSIMD_ARM_AARCH64)
 		// SIMD version.
 		if (!nosimd && (arraylen >= (CHARSIMDSIZE * 2))) {
 			sub_unsigned_char_5_simd(arraylen, data1, data2);
@@ -494,7 +501,7 @@ signed int sub_unsigned_char_6(Py_ssize_t arraylen, int nosimd, unsigned char *d
 	// Math error checking disabled.
 	if (ignoreerrors) {
 
-#if defined(AF_HASSIMD_ARM)
+#if defined(AF_HASSIMD_ARMv7_32BIT) || defined(AF_HASSIMD_ARM_AARCH64)
 		// SIMD version.
 		if (!nosimd && (arraylen >= (CHARSIMDSIZE * 2))) {
 			sub_unsigned_char_6_simd(arraylen, data1, data2, data3);
@@ -535,7 +542,7 @@ signed int sub_signed_short_1(Py_ssize_t arraylen, int nosimd, signed short *dat
 	// Math error checking disabled.
 	if (ignoreerrors) {
 
-#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARM)
+#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARMv7_32BIT) || defined(AF_HASSIMD_ARM_AARCH64)
 		// SIMD version.
 		if (!nosimd && (arraylen >= (SHORTSIMDSIZE * 2))) {
 			sub_signed_short_1_simd(arraylen, data1, param);
@@ -581,7 +588,7 @@ signed int sub_signed_short_2(Py_ssize_t arraylen, int nosimd, signed short *dat
 	// Math error checking disabled.
 	if (ignoreerrors) {
 
-#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARM)
+#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARMv7_32BIT) || defined(AF_HASSIMD_ARM_AARCH64)
 		// SIMD version.
 		if (!nosimd && (arraylen >= (SHORTSIMDSIZE * 2))) {
 			sub_signed_short_2_simd(arraylen, data1, param, data3);
@@ -628,7 +635,7 @@ signed int sub_signed_short_3(Py_ssize_t arraylen, int nosimd, signed short para
 	// Math error checking disabled.
 	if (ignoreerrors) {
 
-#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARM)
+#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARMv7_32BIT) || defined(AF_HASSIMD_ARM_AARCH64)
 		// SIMD version.
 		if (!nosimd && (arraylen >= (SHORTSIMDSIZE * 2))) {
 			sub_signed_short_3_simd(arraylen, param, data2);
@@ -677,7 +684,7 @@ signed int sub_signed_short_4(Py_ssize_t arraylen, int nosimd, signed short para
 	// Math error checking disabled.
 	if (ignoreerrors) {
 
-#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARM)
+#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARMv7_32BIT) || defined(AF_HASSIMD_ARM_AARCH64)
 		// SIMD version.
 		if (!nosimd && (arraylen >= (SHORTSIMDSIZE * 2))) {
 			sub_signed_short_4_simd(arraylen, param, data2, data3);
@@ -727,7 +734,7 @@ signed int sub_signed_short_5(Py_ssize_t arraylen, int nosimd, signed short *dat
 	// Math error checking disabled.
 	if (ignoreerrors) {
 
-#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARM)
+#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARMv7_32BIT) || defined(AF_HASSIMD_ARM_AARCH64)
 		// SIMD version.
 		if (!nosimd && (arraylen >= (SHORTSIMDSIZE * 2))) {
 			sub_signed_short_5_simd(arraylen, data1, data2);
@@ -759,7 +766,7 @@ signed int sub_signed_short_6(Py_ssize_t arraylen, int nosimd, signed short *dat
 	// Math error checking disabled.
 	if (ignoreerrors) {
 
-#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARM)
+#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARMv7_32BIT) || defined(AF_HASSIMD_ARM_AARCH64)
 		// SIMD version.
 		if (!nosimd && (arraylen >= (SHORTSIMDSIZE * 2))) {
 			sub_signed_short_6_simd(arraylen, data1, data2, data3);
@@ -800,7 +807,7 @@ signed int sub_unsigned_short_1(Py_ssize_t arraylen, int nosimd, unsigned short
 	// Math error checking disabled.
 	if (ignoreerrors) {
 
-#if defined(AF_HASSIMD_ARM)
+#if defined(AF_HASSIMD_ARMv7_32BIT) || defined(AF_HASSIMD_ARM_AARCH64)
 		// SIMD version.
 		if (!nosimd && (arraylen >= (SHORTSIMDSIZE * 2))) {
 			sub_unsigned_short_1_simd(arraylen, data1, param);
@@ -831,7 +838,7 @@ signed int sub_unsigned_short_2(Py_ssize_t arraylen, int nosimd, unsigned short
 	// Math error checking disabled.
 	if (ignoreerrors) {
 
-#if defined(AF_HASSIMD_ARM)
+#if defined(AF_HASSIMD_ARMv7_32BIT) || defined(AF_HASSIMD_ARM_AARCH64)
 		// SIMD version.
 		if (!nosimd && (arraylen >= (SHORTSIMDSIZE * 2))) {
 			sub_unsigned_short_2_simd(arraylen, data1, param, data3);
@@ -862,7 +869,7 @@ signed int sub_unsigned_short_3(Py_ssize_t arraylen, int nosimd, unsigned short
 	// Math error checking disabled.
 	if (ignoreerrors) {
 
-#if defined(AF_HASSIMD_ARM)
+#if defined(AF_HASSIMD_ARMv7_32BIT) || defined(AF_HASSIMD_ARM_AARCH64)
 		// SIMD version.
 		if (!nosimd && (arraylen >= (SHORTSIMDSIZE * 2))) {
 			sub_unsigned_short_3_simd(arraylen, param, data2);
@@ -893,7 +900,7 @@ signed int sub_unsigned_short_4(Py_ssize_t arraylen, int nosimd, unsigned short
 	// Math error checking disabled.
 	if (ignoreerrors) {
 
-#if defined(AF_HASSIMD_ARM)
+#if defined(AF_HASSIMD_ARMv7_32BIT) || defined(AF_HASSIMD_ARM_AARCH64)
 		// SIMD version.
 		if (!nosimd && (arraylen >= (SHORTSIMDSIZE * 2))) {
 			sub_unsigned_short_4_simd(arraylen, param, data2, data3);
@@ -926,7 +933,7 @@ signed int sub_unsigned_short_5(Py_ssize_t arraylen, int nosimd, unsigned short
 	// Math error checking disabled.
 	if (ignoreerrors) {
 
-#if defined(AF_HASSIMD_ARM)
+#if defined(AF_HASSIMD_ARMv7_32BIT) || defined(AF_HASSIMD_ARM_AARCH64)
 		// SIMD version.
 		if (!nosimd && (arraylen >= (SHORTSIMDSIZE * 2))) {
 			sub_unsigned_short_5_simd(arraylen, data1, data2);
@@ -957,7 +964,7 @@ signed int sub_unsigned_short_6(Py_ssize_t arraylen, int nosimd, unsigned short
 	// Math error checking disabled.
 	if (ignoreerrors) {
 
-#if defined(AF_HASSIMD_ARM)
+#if defined(AF_HASSIMD_ARMv7_32BIT) || defined(AF_HASSIMD_ARM_AARCH64)
 		// SIMD version.
 		if (!nosimd && (arraylen >= (SHORTSIMDSIZE * 2))) {
 			sub_unsigned_short_6_simd(arraylen, data1, data2, data3);
@@ -998,7 +1005,7 @@ signed int sub_signed_int_1(Py_ssize_t arraylen, int nosimd, signed int *data1,
 	// Math error checking disabled.
 	if (ignoreerrors) {
 
-#if defined(AF_HASSIMD_X86)
+#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARM_AARCH64)
 		// SIMD version.
 		if (!nosimd && (arraylen >= (INTSIMDSIZE * 2))) {
 			sub_signed_int_1_simd(arraylen, data1, param);
@@ -1044,7 +1051,7 @@ signed int sub_signed_int_2(Py_ssize_t arraylen, int nosimd, signed int *data1,
 	// Math error checking disabled.
 	if (ignoreerrors) {
 
-#if defined(AF_HASSIMD_X86)
+#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARM_AARCH64)
 		// SIMD version.
 		if (!nosimd && (arraylen >= (INTSIMDSIZE * 2))) {
 			sub_signed_int_2_simd(arraylen, data1, param, data3);
@@ -1091,7 +1098,7 @@ signed int sub_signed_int_3(Py_ssize_t arraylen, int nosimd, signed int param, s
 	// Math error checking disabled.
 	if (ignoreerrors) {
 
-#if defined(AF_HASSIMD_X86)
+#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARM_AARCH64)
 		// SIMD version.
 		if (!nosimd && (arraylen >= (INTSIMDSIZE * 2))) {
 			sub_signed_int_3_simd(arraylen, param, data2);
@@ -1140,7 +1147,7 @@ signed int sub_signed_int_4(Py_ssize_t arraylen, int nosimd, signed int param, s
 	// Math error checking disabled.
 	if (ignoreerrors) {
 
-#if defined(AF_HASSIMD_X86)
+#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARM_AARCH64)
 		// SIMD version.
 		if (!nosimd && (arraylen >= (INTSIMDSIZE * 2))) {
 			sub_signed_int_4_simd(arraylen, param, data2, data3);
@@ -1190,7 +1197,7 @@ signed int sub_signed_int_5(Py_ssize_t arraylen, int nosimd, signed int *data1,
 	// Math error checking disabled.
 	if (ignoreerrors) {
 
-#if defined(AF_HASSIMD_X86)
+#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARM_AARCH64)
 		// SIMD version.
 		if (!nosimd && (arraylen >= (INTSIMDSIZE * 2))) {
 			sub_signed_int_5_simd(arraylen, data1, data2);
@@ -1222,7 +1229,7 @@ signed int sub_signed_int_6(Py_ssize_t arraylen, int nosimd, signed int *data1,
 	// Math error checking disabled.
 	if (ignoreerrors) {
 
-#if defined(AF_HASSIMD_X86)
+#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARM_AARCH64)
 		// SIMD version.
 		if (!nosimd && (arraylen >= (INTSIMDSIZE * 2))) {
 			sub_signed_int_6_simd(arraylen, data1, data2, data3);
@@ -1255,7 +1262,7 @@ signed int sub_signed_int_6(Py_ssize_t arraylen, int nosimd, signed int *data1,
    ignoreerrors = If true, disable arithmetic math error checking (default is false).
 */
 // param_arr_num_none
-signed int sub_unsigned_int_1(Py_ssize_t arraylen, unsigned int *data1, unsigned int param, unsigned int ignoreerrors) {
+signed int sub_unsigned_int_1(Py_ssize_t arraylen, int nosimd, unsigned int *data1, unsigned int param, unsigned int ignoreerrors) {
 
 	// array index counter.
 	Py_ssize_t x;
@@ -1263,6 +1270,14 @@ signed int sub_unsigned_int_1(Py_ssize_t arraylen, unsigned int *data1, unsigned
 	// Math error checking disabled.
 	if (ignoreerrors) {
 
+#if defined(AF_HASSIMD_ARM_AARCH64)
+		// SIMD version.
+		if (!nosimd && (arraylen >= (INTSIMDSIZE * 2))) {
+			sub_unsigned_int_1_simd(arraylen, data1, param);
+			return ARR_NO_ERR;
+		}
+#endif
+
 		for (x = 0; x < arraylen; x++) {
 			data1[x] = data1[x] - param;
 		}
@@ -1278,7 +1293,7 @@ signed int sub_unsigned_int_1(Py_ssize_t arraylen, unsigned int *data1, unsigned
 }
 
 // param_arr_num_arr
-signed int sub_unsigned_int_2(Py_ssize_t arraylen, unsigned int *data1, unsigned int param, unsigned int *data3, unsigned int ignoreerrors) {
+signed int sub_unsigned_int_2(Py_ssize_t arraylen, int nosimd, unsigned int *data1, unsigned int param, unsigned int *data3, unsigned int ignoreerrors) {
 
 	// array index counter.
 	Py_ssize_t x;
@@ -1286,6 +1301,14 @@ signed int sub_unsigned_int_2(Py_ssize_t arraylen, unsigned int *data1, unsigned
 	// Math error checking disabled.
 	if (ignoreerrors) {
 
+#if defined(AF_HASSIMD_ARM_AARCH64)
+		// SIMD version.
+		if (!nosimd && (arraylen >= (INTSIMDSIZE * 2))) {
+			sub_unsigned_int_2_simd(arraylen, data1, param, data3);
+			return ARR_NO_ERR;
+		}
+#endif
+
 		for (x = 0; x < arraylen; x++) {
 			data3[x] = data1[x] - param;
 		}
@@ -1301,7 +1324,7 @@ signed int sub_unsigned_int_2(Py_ssize_t arraylen, unsigned int *data1, unsigned
 }
 
 // param_num_arr_none
-signed int sub_unsigned_int_3(Py_ssize_t arraylen, unsigned int param, unsigned int *data2, unsigned int ignoreerrors) {
+signed int sub_unsigned_int_3(Py_ssize_t arraylen, int nosimd, unsigned int param, unsigned int *data2, unsigned int ignoreerrors) {
 
 	// array index counter.
 	Py_ssize_t x;
@@ -1309,6 +1332,14 @@ signed int sub_unsigned_int_3(Py_ssize_t arraylen, unsigned int param, unsigned
 	// Math error checking disabled.
 	if (ignoreerrors) {
 
+#if defined(AF_HASSIMD_ARM_AARCH64)
+		// SIMD version.
+		if (!nosimd && (arraylen >= (INTSIMDSIZE * 2))) {
+			sub_unsigned_int_3_simd(arraylen, param, data2);
+			return ARR_NO_ERR;
+		}
+#endif
+
 		for (x = 0; x < arraylen; x++) {
 			data2[x] = param - data2[x];
 		}
@@ -1324,7 +1355,7 @@ signed int sub_unsigned_int_3(Py_ssize_t arraylen, unsigned int param, unsigned
 }
 
 // param_num_arr_arr
-signed int sub_unsigned_int_4(Py_ssize_t arraylen, unsigned int param, unsigned int *data2, unsigned int *data3, unsigned int ignoreerrors) {
+signed int sub_unsigned_int_4(Py_ssize_t arraylen, int nosimd, unsigned int param, unsigned int *data2, unsigned int *data3, unsigned int ignoreerrors) {
 
 	// array index counter.
 	Py_ssize_t x;
@@ -1332,6 +1363,14 @@ signed int sub_unsigned_int_4(Py_ssize_t arraylen, unsigned int param, unsigned
 	// Math error checking disabled.
 	if (ignoreerrors) {
 
+#if defined(AF_HASSIMD_ARM_AARCH64)
+		// SIMD version.
+		if (!nosimd && (arraylen >= (INTSIMDSIZE * 2))) {
+			sub_unsigned_int_4_simd(arraylen, param, data2, data3);
+			return ARR_NO_ERR;
+		}
+#endif
+
 		for (x = 0; x < arraylen; x++) {
 			data3[x] = param - data2[x];
 		}
@@ -1349,7 +1388,7 @@ signed int sub_unsigned_int_4(Py_ssize_t arraylen, unsigned int param, unsigned
 
 
 // param_arr_arr_none
-signed int sub_unsigned_int_5(Py_ssize_t arraylen, unsigned int *data1, unsigned int *data2, unsigned int ignoreerrors) {
+signed int sub_unsigned_int_5(Py_ssize_t arraylen, int nosimd, unsigned int *data1, unsigned int *data2, unsigned int ignoreerrors) {
 
 	// array index counter.
 	Py_ssize_t x;
@@ -1357,6 +1396,14 @@ signed int sub_unsigned_int_5(Py_ssize_t arraylen, unsigned int *data1, unsigned
 	// Math error checking disabled.
 	if (ignoreerrors) {
 
+#if defined(AF_HASSIMD_ARM_AARCH64)
+		// SIMD version.
+		if (!nosimd && (arraylen >= (INTSIMDSIZE * 2))) {
+			sub_unsigned_int_5_simd(arraylen, data1, data2);
+			return ARR_NO_ERR;
+		}
+#endif
+
 		for (x = 0; x < arraylen; x++) {
 			data1[x] = data1[x] - data2[x];
 		}
@@ -1372,7 +1419,7 @@ signed int sub_unsigned_int_5(Py_ssize_t arraylen, unsigned int *data1, unsigned
 }
 
 // param_arr_arr_arr
-signed int sub_unsigned_int_6(Py_ssize_t arraylen, unsigned int *data1, unsigned int *data2, unsigned int *data3, unsigned int ignoreerrors) {
+signed int sub_unsigned_int_6(Py_ssize_t arraylen, int nosimd, unsigned int *data1, unsigned int *data2, unsigned int *data3, unsigned int ignoreerrors) {
 
 	// array index counter.
 	Py_ssize_t x;
@@ -1380,6 +1427,14 @@ signed int sub_unsigned_int_6(Py_ssize_t arraylen, unsigned int *data1, unsigned
 	// Math error checking disabled.
 	if (ignoreerrors) {
 
+#if defined(AF_HASSIMD_ARM_AARCH64)
+		// SIMD version.
+		if (!nosimd && (arraylen >= (INTSIMDSIZE * 2))) {
+			sub_unsigned_int_6_simd(arraylen, data1, data2, data3);
+			return ARR_NO_ERR;
+		}
+#endif
+
 		for (x = 0; x < arraylen; x++) {
 			data3[x] = data1[x] - data2[x];
 		}
@@ -2146,7 +2201,7 @@ signed int sub_float_1(Py_ssize_t arraylen, int nosimd, float *data1, float para
 	// Math error checking disabled.
 	if (ignoreerrors) {
 
-#if defined(AF_HASSIMD_X86)
+#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARM_AARCH64)
 		// SIMD version.
 		if (!nosimd && (arraylen >= (FLOATSIMDSIZE * 2))) {
 			sub_float_1_simd(arraylen, data1, param);
@@ -2177,7 +2232,7 @@ signed int sub_float_2(Py_ssize_t arraylen, int nosimd, float *data1, float para
 	// Math error checking disabled.
 	if (ignoreerrors) {
 
-#if defined(AF_HASSIMD_X86)
+#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARM_AARCH64)
 		// SIMD version.
 		if (!nosimd && (arraylen >= (FLOATSIMDSIZE * 2))) {
 			sub_float_2_simd(arraylen, data1, param, data3);
@@ -2208,7 +2263,7 @@ signed int sub_float_3(Py_ssize_t arraylen, int nosimd, float param, float *data
 	// Math error checking disabled.
 	if (ignoreerrors) {
 
-#if defined(AF_HASSIMD_X86)
+#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARM_AARCH64)
 		// SIMD version.
 		if (!nosimd && (arraylen >= (FLOATSIMDSIZE * 2))) {
 			sub_float_3_simd(arraylen, param, data2);
@@ -2239,7 +2294,7 @@ signed int sub_float_4(Py_ssize_t arraylen, int nosimd, float param, float *data
 	// Math error checking disabled.
 	if (ignoreerrors) {
 
-#if defined(AF_HASSIMD_X86)
+#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARM_AARCH64)
 		// SIMD version.
 		if (!nosimd && (arraylen >= (FLOATSIMDSIZE * 2))) {
 			sub_float_4_simd(arraylen, param, data2, data3);
@@ -2272,7 +2327,7 @@ signed int sub_float_5(Py_ssize_t arraylen, int nosimd, float *data1, float *dat
 	// Math error checking disabled.
 	if (ignoreerrors) {
 
-#if defined(AF_HASSIMD_X86)
+#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARM_AARCH64)
 		// SIMD version.
 		if (!nosimd && (arraylen >= (FLOATSIMDSIZE * 2))) {
 			sub_float_5_simd(arraylen, data1, data2);
@@ -2303,7 +2358,7 @@ signed int sub_float_6(Py_ssize_t arraylen, int nosimd, float *data1, float *dat
 	// Math error checking disabled.
 	if (ignoreerrors) {
 
-#if defined(AF_HASSIMD_X86)
+#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARM_AARCH64)
 		// SIMD version.
 		if (!nosimd && (arraylen >= (FLOATSIMDSIZE * 2))) {
 			sub_float_6_simd(arraylen, data1, data2, data3);
@@ -2708,27 +2763,27 @@ static PyObject *py_sub(PyObject *self, PyObject *args, PyObject *keywds) {
 		case 'I' : {
 			switch (arraydata.paramcat) {
 				case param_arr_num_none : {
-					resultcode = sub_unsigned_int_1(arraydata.arraylength, arraydata.array1.I, arraydata.param.I, arraydata.ignoreerrors);
+					resultcode = sub_unsigned_int_1(arraydata.arraylength, arraydata.nosimd, arraydata.array1.I, arraydata.param.I, arraydata.ignoreerrors);
 					break;
 				}
 				case param_arr_num_arr : {
-					resultcode = sub_unsigned_int_2(arraydata.arraylength, arraydata.array1.I, arraydata.param.I, arraydata.array3.I, arraydata.ignoreerrors);
+					resultcode = sub_unsigned_int_2(arraydata.arraylength, arraydata.nosimd, arraydata.array1.I, arraydata.param.I, arraydata.array3.I, arraydata.ignoreerrors);
 					break;
 				}
 				case param_num_arr_none : {
-					resultcode = sub_unsigned_int_3(arraydata.arraylength, arraydata.param.I, arraydata.array2.I, arraydata.ignoreerrors);
+					resultcode = sub_unsigned_int_3(arraydata.arraylength, arraydata.nosimd, arraydata.param.I, arraydata.array2.I, arraydata.ignoreerrors);
 					break;
 				}
 				case param_num_arr_arr : {
-					resultcode = sub_unsigned_int_4(arraydata.arraylength, arraydata.param.I, arraydata.array2.I, arraydata.array3.I, arraydata.ignoreerrors);
+					resultcode = sub_unsigned_int_4(arraydata.arraylength, arraydata.nosimd, arraydata.param.I, arraydata.array2.I, arraydata.array3.I, arraydata.ignoreerrors);
 					break;
 				}
 				case param_arr_arr_none : {
-					resultcode = sub_unsigned_int_5(arraydata.arraylength, arraydata.array1.I, arraydata.array2.I, arraydata.ignoreerrors);
+					resultcode = sub_unsigned_int_5(arraydata.arraylength, arraydata.nosimd, arraydata.array1.I, arraydata.array2.I, arraydata.ignoreerrors);
 					break;
 				}
 				case param_arr_arr_arr : {
-					resultcode = sub_unsigned_int_6(arraydata.arraylength, arraydata.array1.I, arraydata.array2.I, arraydata.array3.I, arraydata.ignoreerrors);
+					resultcode = sub_unsigned_int_6(arraydata.arraylength, arraydata.nosimd, arraydata.array1.I, arraydata.array2.I, arraydata.array3.I, arraydata.ignoreerrors);
 					break;
 				}
 			}
