@@ -7,7 +7,7 @@
 //
 //------------------------------------------------------------------------------
 //
-//   Copyright 2014 - 2018    Michael Griffin    <m12.griffin@gmail.com>
+//   Copyright 2014 - 2020    Michael Griffin    <m12.griffin@gmail.com>
 //
 //   Licensed under the Apache License, Version 2.0 (the "License");
 //   you may not use this file except in compliance with the License.
@@ -48,9 +48,16 @@
 #include "xor_simd_x86.h"
 #endif
 
-#ifdef AF_HASSIMD_ARM
+#if defined(AF_HASSIMD_ARMv7_32BIT) || defined(AF_HASSIMD_ARM_AARCH64)
 #include "arm_neon.h"
-#include "xor_simd_arm.h"
+#endif
+
+#if defined(AF_HASSIMD_ARMv7_32BIT)
+#include "xor_simd_armv7.h"
+#endif
+
+#if defined(AF_HASSIMD_ARM_AARCH64)
+#include "xor_simd_armv8.h"
 #endif
 
 /*--------------------------------------------------------------------------- */
@@ -71,7 +78,7 @@ void xor_signed_char_1(Py_ssize_t arraylen, int nosimd, signed char *data1, sign
 	// array index counter.
 	Py_ssize_t x;
 
-#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARM)
+#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARMv7_32BIT) || defined(AF_HASSIMD_ARM_AARCH64)
 	// SIMD version.
 	if (!nosimd && (arraylen >= (CHARSIMDSIZE * 2))) {
 		xor_signed_char_1_simd(arraylen, data1, param);
@@ -92,7 +99,7 @@ void xor_signed_char_2(Py_ssize_t arraylen, int nosimd, signed char *data1, sign
 	// array index counter.
 	Py_ssize_t x;
 
-#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARM)
+#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARMv7_32BIT) || defined(AF_HASSIMD_ARM_AARCH64)
 	// SIMD version.
 	if (!nosimd && (arraylen >= (CHARSIMDSIZE * 2))) {
 		xor_signed_char_2_simd(arraylen, data1, param, data3);
@@ -112,7 +119,7 @@ void xor_signed_char_3(Py_ssize_t arraylen, int nosimd, signed char param, signe
 	// array index counter.
 	Py_ssize_t x;
 
-#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARM)
+#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARMv7_32BIT) || defined(AF_HASSIMD_ARM_AARCH64)
 	// SIMD version.
 	if (!nosimd && (arraylen >= (CHARSIMDSIZE * 2))) {
 		xor_signed_char_3_simd(arraylen, param, data2);
@@ -132,7 +139,7 @@ void xor_signed_char_4(Py_ssize_t arraylen, int nosimd, signed char param, signe
 	// array index counter.
 	Py_ssize_t x;
 
-#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARM)
+#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARMv7_32BIT) || defined(AF_HASSIMD_ARM_AARCH64)
 	// SIMD version.
 	if (!nosimd && (arraylen >= (CHARSIMDSIZE * 2))) {
 		xor_signed_char_4_simd(arraylen, param, data2, data3);
@@ -154,7 +161,7 @@ void xor_signed_char_5(Py_ssize_t arraylen, int nosimd, signed char *data1, sign
 	// array index counter.
 	Py_ssize_t x;
 
-#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARM)
+#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARMv7_32BIT) || defined(AF_HASSIMD_ARM_AARCH64)
 	// SIMD version.
 	if (!nosimd && (arraylen >= (CHARSIMDSIZE * 2))) {
 		xor_signed_char_5_simd(arraylen, data1, data2);
@@ -174,7 +181,7 @@ void xor_signed_char_6(Py_ssize_t arraylen, int nosimd, signed char *data1, sign
 	// array index counter.
 	Py_ssize_t x;
 
-#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARM)
+#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARMv7_32BIT) || defined(AF_HASSIMD_ARM_AARCH64)
 	// SIMD version.
 	if (!nosimd && (arraylen >= (CHARSIMDSIZE * 2))) {
 		xor_signed_char_6_simd(arraylen, data1, data2, data3);
@@ -203,7 +210,7 @@ void xor_unsigned_char_1(Py_ssize_t arraylen, int nosimd, unsigned char *data1,
 	// array index counter.
 	Py_ssize_t x;
 
-#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARM)
+#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARMv7_32BIT) || defined(AF_HASSIMD_ARM_AARCH64)
 	// SIMD version.
 	if (!nosimd && (arraylen >= (CHARSIMDSIZE * 2))) {
 		xor_unsigned_char_1_simd(arraylen, data1, param);
@@ -224,7 +231,7 @@ void xor_unsigned_char_2(Py_ssize_t arraylen, int nosimd, unsigned char *data1,
 	// array index counter.
 	Py_ssize_t x;
 
-#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARM)
+#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARMv7_32BIT) || defined(AF_HASSIMD_ARM_AARCH64)
 	// SIMD version.
 	if (!nosimd && (arraylen >= (CHARSIMDSIZE * 2))) {
 		xor_unsigned_char_2_simd(arraylen, data1, param, data3);
@@ -244,7 +251,7 @@ void xor_unsigned_char_3(Py_ssize_t arraylen, int nosimd, unsigned char param, u
 	// array index counter.
 	Py_ssize_t x;
 
-#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARM)
+#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARMv7_32BIT) || defined(AF_HASSIMD_ARM_AARCH64)
 	// SIMD version.
 	if (!nosimd && (arraylen >= (CHARSIMDSIZE * 2))) {
 		xor_unsigned_char_3_simd(arraylen, param, data2);
@@ -264,7 +271,7 @@ void xor_unsigned_char_4(Py_ssize_t arraylen, int nosimd, unsigned char param, u
 	// array index counter.
 	Py_ssize_t x;
 
-#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARM)
+#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARMv7_32BIT) || defined(AF_HASSIMD_ARM_AARCH64)
 	// SIMD version.
 	if (!nosimd && (arraylen >= (CHARSIMDSIZE * 2))) {
 		xor_unsigned_char_4_simd(arraylen, param, data2, data3);
@@ -286,7 +293,7 @@ void xor_unsigned_char_5(Py_ssize_t arraylen, int nosimd, unsigned char *data1,
 	// array index counter.
 	Py_ssize_t x;
 
-#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARM)
+#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARMv7_32BIT) || defined(AF_HASSIMD_ARM_AARCH64)
 	// SIMD version.
 	if (!nosimd && (arraylen >= (CHARSIMDSIZE * 2))) {
 		xor_unsigned_char_5_simd(arraylen, data1, data2);
@@ -306,7 +313,7 @@ void xor_unsigned_char_6(Py_ssize_t arraylen, int nosimd, unsigned char *data1,
 	// array index counter.
 	Py_ssize_t x;
 
-#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARM)
+#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARMv7_32BIT) || defined(AF_HASSIMD_ARM_AARCH64)
 	// SIMD version.
 	if (!nosimd && (arraylen >= (CHARSIMDSIZE * 2))) {
 		xor_unsigned_char_6_simd(arraylen, data1, data2, data3);
@@ -335,7 +342,7 @@ void xor_signed_short_1(Py_ssize_t arraylen, int nosimd, signed short *data1, si
 	// array index counter.
 	Py_ssize_t x;
 
-#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARM)
+#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARMv7_32BIT) || defined(AF_HASSIMD_ARM_AARCH64)
 	// SIMD version.
 	if (!nosimd && (arraylen >= (SHORTSIMDSIZE * 2))) {
 		xor_signed_short_1_simd(arraylen, data1, param);
@@ -356,7 +363,7 @@ void xor_signed_short_2(Py_ssize_t arraylen, int nosimd, signed short *data1, si
 	// array index counter.
 	Py_ssize_t x;
 
-#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARM)
+#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARMv7_32BIT) || defined(AF_HASSIMD_ARM_AARCH64)
 	// SIMD version.
 	if (!nosimd && (arraylen >= (SHORTSIMDSIZE * 2))) {
 		xor_signed_short_2_simd(arraylen, data1, param, data3);
@@ -376,7 +383,7 @@ void xor_signed_short_3(Py_ssize_t arraylen, int nosimd, signed short param, sig
 	// array index counter.
 	Py_ssize_t x;
 
-#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARM)
+#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARMv7_32BIT) || defined(AF_HASSIMD_ARM_AARCH64)
 	// SIMD version.
 	if (!nosimd && (arraylen >= (SHORTSIMDSIZE * 2))) {
 		xor_signed_short_3_simd(arraylen, param, data2);
@@ -396,7 +403,7 @@ void xor_signed_short_4(Py_ssize_t arraylen, int nosimd, signed short param, sig
 	// array index counter.
 	Py_ssize_t x;
 
-#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARM)
+#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARMv7_32BIT) || defined(AF_HASSIMD_ARM_AARCH64)
 	// SIMD version.
 	if (!nosimd && (arraylen >= (SHORTSIMDSIZE * 2))) {
 		xor_signed_short_4_simd(arraylen, param, data2, data3);
@@ -418,7 +425,7 @@ void xor_signed_short_5(Py_ssize_t arraylen, int nosimd, signed short *data1, si
 	// array index counter.
 	Py_ssize_t x;
 
-#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARM)
+#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARMv7_32BIT) || defined(AF_HASSIMD_ARM_AARCH64)
 	// SIMD version.
 	if (!nosimd && (arraylen >= (SHORTSIMDSIZE * 2))) {
 		xor_signed_short_5_simd(arraylen, data1, data2);
@@ -438,7 +445,7 @@ void xor_signed_short_6(Py_ssize_t arraylen, int nosimd, signed short *data1, si
 	// array index counter.
 	Py_ssize_t x;
 
-#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARM)
+#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARMv7_32BIT) || defined(AF_HASSIMD_ARM_AARCH64)
 	// SIMD version.
 	if (!nosimd && (arraylen >= (SHORTSIMDSIZE * 2))) {
 		xor_signed_short_6_simd(arraylen, data1, data2, data3);
@@ -467,7 +474,7 @@ void xor_unsigned_short_1(Py_ssize_t arraylen, int nosimd, unsigned short *data1
 	// array index counter.
 	Py_ssize_t x;
 
-#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARM)
+#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARMv7_32BIT) || defined(AF_HASSIMD_ARM_AARCH64)
 	// SIMD version.
 	if (!nosimd && (arraylen >= (SHORTSIMDSIZE * 2))) {
 		xor_unsigned_short_1_simd(arraylen, data1, param);
@@ -488,7 +495,7 @@ void xor_unsigned_short_2(Py_ssize_t arraylen, int nosimd, unsigned short *data1
 	// array index counter.
 	Py_ssize_t x;
 
-#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARM)
+#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARMv7_32BIT) || defined(AF_HASSIMD_ARM_AARCH64)
 	// SIMD version.
 	if (!nosimd && (arraylen >= (SHORTSIMDSIZE * 2))) {
 		xor_unsigned_short_2_simd(arraylen, data1, param, data3);
@@ -508,7 +515,7 @@ void xor_unsigned_short_3(Py_ssize_t arraylen, int nosimd, unsigned short param,
 	// array index counter.
 	Py_ssize_t x;
 
-#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARM)
+#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARMv7_32BIT) || defined(AF_HASSIMD_ARM_AARCH64)
 	// SIMD version.
 	if (!nosimd && (arraylen >= (SHORTSIMDSIZE * 2))) {
 		xor_unsigned_short_3_simd(arraylen, param, data2);
@@ -528,7 +535,7 @@ void xor_unsigned_short_4(Py_ssize_t arraylen, int nosimd, unsigned short param,
 	// array index counter.
 	Py_ssize_t x;
 
-#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARM)
+#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARMv7_32BIT) || defined(AF_HASSIMD_ARM_AARCH64)
 	// SIMD version.
 	if (!nosimd && (arraylen >= (SHORTSIMDSIZE * 2))) {
 		xor_unsigned_short_4_simd(arraylen, param, data2, data3);
@@ -550,7 +557,7 @@ void xor_unsigned_short_5(Py_ssize_t arraylen, int nosimd, unsigned short *data1
 	// array index counter.
 	Py_ssize_t x;
 
-#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARM)
+#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARMv7_32BIT) || defined(AF_HASSIMD_ARM_AARCH64)
 	// SIMD version.
 	if (!nosimd && (arraylen >= (SHORTSIMDSIZE * 2))) {
 		xor_unsigned_short_5_simd(arraylen, data1, data2);
@@ -570,7 +577,7 @@ void xor_unsigned_short_6(Py_ssize_t arraylen, int nosimd, unsigned short *data1
 	// array index counter.
 	Py_ssize_t x;
 
-#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARM)
+#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARMv7_32BIT) || defined(AF_HASSIMD_ARM_AARCH64)
 	// SIMD version.
 	if (!nosimd && (arraylen >= (SHORTSIMDSIZE * 2))) {
 		xor_unsigned_short_6_simd(arraylen, data1, data2, data3);
@@ -599,7 +606,7 @@ void xor_signed_int_1(Py_ssize_t arraylen, int nosimd, signed int *data1, signed
 	// array index counter.
 	Py_ssize_t x;
 
-#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARM)
+#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARMv7_32BIT) || defined(AF_HASSIMD_ARM_AARCH64)
 	// SIMD version.
 	if (!nosimd && (arraylen >= (INTSIMDSIZE * 2))) {
 		xor_signed_int_1_simd(arraylen, data1, param);
@@ -620,7 +627,7 @@ void xor_signed_int_2(Py_ssize_t arraylen, int nosimd, signed int *data1, signed
 	// array index counter.
 	Py_ssize_t x;
 
-#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARM)
+#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARMv7_32BIT) || defined(AF_HASSIMD_ARM_AARCH64)
 	// SIMD version.
 	if (!nosimd && (arraylen >= (INTSIMDSIZE * 2))) {
 		xor_signed_int_2_simd(arraylen, data1, param, data3);
@@ -640,7 +647,7 @@ void xor_signed_int_3(Py_ssize_t arraylen, int nosimd, signed int param, signed
 	// array index counter.
 	Py_ssize_t x;
 
-#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARM)
+#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARMv7_32BIT) || defined(AF_HASSIMD_ARM_AARCH64)
 	// SIMD version.
 	if (!nosimd && (arraylen >= (INTSIMDSIZE * 2))) {
 		xor_signed_int_3_simd(arraylen, param, data2);
@@ -660,7 +667,7 @@ void xor_signed_int_4(Py_ssize_t arraylen, int nosimd, signed int param, signed
 	// array index counter.
 	Py_ssize_t x;
 
-#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARM)
+#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARMv7_32BIT) || defined(AF_HASSIMD_ARM_AARCH64)
 	// SIMD version.
 	if (!nosimd && (arraylen >= (INTSIMDSIZE * 2))) {
 		xor_signed_int_4_simd(arraylen, param, data2, data3);
@@ -682,7 +689,7 @@ void xor_signed_int_5(Py_ssize_t arraylen, int nosimd, signed int *data1, signed
 	// array index counter.
 	Py_ssize_t x;
 
-#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARM)
+#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARMv7_32BIT) || defined(AF_HASSIMD_ARM_AARCH64)
 	// SIMD version.
 	if (!nosimd && (arraylen >= (INTSIMDSIZE * 2))) {
 		xor_signed_int_5_simd(arraylen, data1, data2);
@@ -702,7 +709,7 @@ void xor_signed_int_6(Py_ssize_t arraylen, int nosimd, signed int *data1, signed
 	// array index counter.
 	Py_ssize_t x;
 
-#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARM)
+#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARMv7_32BIT) || defined(AF_HASSIMD_ARM_AARCH64)
 	// SIMD version.
 	if (!nosimd && (arraylen >= (INTSIMDSIZE * 2))) {
 		xor_signed_int_6_simd(arraylen, data1, data2, data3);
@@ -731,7 +738,7 @@ void xor_unsigned_int_1(Py_ssize_t arraylen, int nosimd, unsigned int *data1, un
 	// array index counter.
 	Py_ssize_t x;
 
-#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARM)
+#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARMv7_32BIT) || defined(AF_HASSIMD_ARM_AARCH64)
 	// SIMD version.
 	if (!nosimd && (arraylen >= (INTSIMDSIZE * 2))) {
 		xor_unsigned_int_1_simd(arraylen, data1, param);
@@ -752,7 +759,7 @@ void xor_unsigned_int_2(Py_ssize_t arraylen, int nosimd, unsigned int *data1, un
 	// array index counter.
 	Py_ssize_t x;
 
-#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARM)
+#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARMv7_32BIT) || defined(AF_HASSIMD_ARM_AARCH64)
 	// SIMD version.
 	if (!nosimd && (arraylen >= (INTSIMDSIZE * 2))) {
 		xor_unsigned_int_2_simd(arraylen, data1, param, data3);
@@ -772,7 +779,7 @@ void xor_unsigned_int_3(Py_ssize_t arraylen, int nosimd, unsigned int param, uns
 	// array index counter.
 	Py_ssize_t x;
 
-#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARM)
+#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARMv7_32BIT) || defined(AF_HASSIMD_ARM_AARCH64)
 	// SIMD version.
 	if (!nosimd && (arraylen >= (INTSIMDSIZE * 2))) {
 		xor_unsigned_int_3_simd(arraylen, param, data2);
@@ -792,7 +799,7 @@ void xor_unsigned_int_4(Py_ssize_t arraylen, int nosimd, unsigned int param, uns
 	// array index counter.
 	Py_ssize_t x;
 
-#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARM)
+#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARMv7_32BIT) || defined(AF_HASSIMD_ARM_AARCH64)
 	// SIMD version.
 	if (!nosimd && (arraylen >= (INTSIMDSIZE * 2))) {
 		xor_unsigned_int_4_simd(arraylen, param, data2, data3);
@@ -814,7 +821,7 @@ void xor_unsigned_int_5(Py_ssize_t arraylen, int nosimd, unsigned int *data1, un
 	// array index counter.
 	Py_ssize_t x;
 
-#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARM)
+#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARMv7_32BIT) || defined(AF_HASSIMD_ARM_AARCH64)
 	// SIMD version.
 	if (!nosimd && (arraylen >= (INTSIMDSIZE * 2))) {
 		xor_unsigned_int_5_simd(arraylen, data1, data2);
@@ -834,7 +841,7 @@ void xor_unsigned_int_6(Py_ssize_t arraylen, int nosimd, unsigned int *data1, un
 	// array index counter.
 	Py_ssize_t x;
 
-#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARM)
+#if defined(AF_HASSIMD_X86) || defined(AF_HASSIMD_ARMv7_32BIT) || defined(AF_HASSIMD_ARM_AARCH64)
 	// SIMD version.
 	if (!nosimd && (arraylen >= (INTSIMDSIZE * 2))) {
 		xor_unsigned_int_6_simd(arraylen, data1, data2, data3);
